# -*- coding: utf-8 -*-
import datetime
from django.core.paginator import Paginator
from django.utils.translation import ugettext as _
from django.conf import settings
from django.db.models import Q

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.content_file.utils import get_querystring_dict, get_first_day_of_week, get_first_day_of_month, get_first_day_of_year
from jmb.content_file.models import ContentFile
from jmb.content_file.cms.models import ContentFileSettings


class CMSPluginExtended(CMSPluginBase):

    def get_dict_from_querystring(self, request):
        dictionary = {}
        if request.GET:
            for key in request.GET.keys():
                if request.GET.get(key) != "":
                    dictionary[key] = request.GET.get(key)
        return dictionary

    def get_querystring_from_dictionary(self, dictionary):
        query_list = []
        for key, value in dictionary.items():
            if key != 'page' and key != 'obj_page' and value != '':
                query_list.append(key + "=" + value)
        return "&".join(query_list)

    def filter_queryset(self, queryset, filter_dict):
        q_objects = Q()
        for key, value in filter_dict.items():
            q_objects |= Q(**{key: value})
        try:
            queryset = queryset.filter(q_objects)
        except Exception, e:
            print e
        return queryset


class ContentFilesPlugin(CMSPluginExtended):
    model = ContentFileSettings
    name = _("Content Files List")
    module = _("ContentFile")
    admin_preview = False

    def render(self, context, instance, placeholder):
        self.render_template = "content_file/cms_plugins/empty.html"
        ordering = instance.ordering.lower().replace(" ", "").split(",")
        objects = ContentFile.objects.filter(status=1)
        query_string = None
        request = context['request']
        page = 1

        if 'paginator_instance_slug' in request.GET and request.GET['paginator_instance_slug'] == instance.slug:
            page = int(request.GET['page'])

        querystring_dict = get_querystring_dict(request)
        today = datetime.datetime.today()

        if instance.status == 0:
            return context

        ############################################################################################
        # Check per utenti non autenticati
        ############################################################################################
        if instance.show_if_authenticated == "authenticated" and not request.user.is_authenticated():
            return context
        elif instance.show_if_authenticated == "not_authenticated" and request.user.is_authenticated():
            return context

        self.render_template = instance.template_name

        ############################################################################################
        # Filtri elementi in funzione della data di pubblicazione
        ############################################################################################

        # pub_date_begin
        objects.filter(Q(pub_date__lte=today)|Q(pub_date=None))

        ############################################################################################
        # Filtri simple e advanced search
        ############################################################################################

        # Simple searched
        objects = objects.filter(self.get_q_simple_search(request, instance))

        # Advanced search
        objects = objects.filter(self.get_q_advanced_search(request, instance))

        ############################################################################################
        # Ordinamento, limite e paginator
        ############################################################################################

        # Ordinamento
        if ordering:
            objects = objects.order_by(*ordering)

        # Limite
        if instance.objects_limit:
            objects = objects[:instance.objects_limit]

        # Paginator
        paginator = Paginator(objects, instance.objects_per_page)
        objects = paginator.page(page)
        adjacent_pages = instance.adjacent_pages
        page_numbers = [n for n in range(page - adjacent_pages, page + adjacent_pages + 1) if n > 0 and n <= paginator.num_pages]
        show_first = True if 1 not in page_numbers else False
        show_last = True if paginator.num_pages not in page_numbers else False

        context.update({
            'placeholder': placeholder,
            'instance': instance,
            'user': request.user,
            'page': page,
            'prev_page': page - 1,
            'next_page': page + 1,
            'show_first': show_first,
            'show_last': show_last,
            'objects': objects,
            'page_numbers': page_numbers,
            'paginator': paginator,
        })
        return context

    def get_q_simple_search(self, request, contentfile_setting):
        q_objects = Q()
        if (
            request.method == 'GET'
            and 'contentfile_search_submit' in request.GET
            and request.GET.get('contentfile_slug', None) == str(contentfile_setting.slug)
        ):
            # Testo libero
            q_words = [word for word in request.GET['simple-search-input'].split(",")]
            search_lookups = [
                'short_description__icontains',
            ]
            for q_word in q_words:
                for lookup in search_lookups:
                    q_objects |= Q(**{lookup: q_word})
        return q_objects

    def get_q_advanced_search(self, request, contentfile_setting):
        q_objects = Q()
        if (
            request.method == 'GET'
            and 'advanced_serch_submit' in request.GET
            and request.GET.get('contentfile_slug', None) == str(contentfile_setting.slug)
        ):
            query_dict = dict(
                titolo=['title__icontains'],
                id=['id'],
                data_inizio=['date_content__gte'],
                data_fine=['date_content__lte'],
            )
            for key in request.GET:
                if key not in ["allwords", "onlytheseswords", "oneword"]:
                    if key in query_dict:
                        for value in request.GET.getlist(key):
                            value = value.strip()
                            if value != "":
                                for lookup in query_dict[key]:
                                    q_objects &= Q(**{lookup: value})
                    else:
                        if key == "has_foto":
                            q_objects &= Q(**{'small_image__isnull': False})
                else:
                    search_lookups = [
                        'short_description__icontains',
                    ]
                    # Tutte queste parole
                    if key == "allwords":
                        words = request.GET["allwords"].split(",")
                        for word in words:
                            word = word.strip()
                            if word != "":
                                for lookup in search_lookups:
                                    q_objects &= Q(**{lookup: word})
                    # Questa esatta parola o frase
                    elif key == "onlytheseswords":
                        word = request.GET["onlytheseswords"].strip()
                        if word != "":
                            for lookup in search_lookups:
                                q_objects &= Q(**{lookup: word})
                    # Una qualunque di queste parole
                    elif key == "oneword":
                        words = request.GET["oneword"].split(",")
                        for word in words:
                            word = word.strip()
                            if word != "":
                                for lookup in search_lookups:
                                    q_objects |= Q(**{lookup: word})
        return q_objects

plugin_pool.register_plugin(ContentFilesPlugin)
